clear
% Create structure array of useful handles.
handles = struct(...
    'pat',   [],             'save',    [],            'getstring',    [],    ...
    'pts',  zeros(1,4),      'mks',       zeros(1,4),  'txs',     zeros(1,4), ...
    'aim',  [],             'indicator', zeros(1,3),                         ...
    'reset', [],             'exact',          [],                            ...
    'rotate_left',  [],      'rotate_right',   [],     'life',         [],    ...
    'slider_f',     [],      'slider_sig_len', [],     'fix',          [],    ...
    'f_min',        [],      'f_max',   [],            'f_cur',        [],    ...
    'sig_len_min',  [],      'sig_len_max',    [],     'sig_len_cur',  [],    ...
    'frame',        [],      'info_string',    [],     'num_pts', zeros(1,4), ...
    'text_sig_len_cur', [],  'text_f_cur',     [],     'mode',    zeros(1,3));
% ============================================================
% Draw Header Figure.
% Prerocessing previous SIGVAR Window (if exist).
num_fig = findobj('Tag','sigvar');
if  length(num_fig)
    set(num_fig,'Units','normalized')
    pos_fig = get(num_fig,'Position');  % Read previous SIGVAR Window Position.
    close(num_fig)                      % Close previous SIGVAR Window.
end
% Create a new figure (standard  MATLAB  figure used).
num_fig = figure( ...
    'Name', 'Signal Properties Variator', ...
    'NextPlot', 'replacechildren', ...
    'NumberTitle', 'of', ...
    'Resize', 'of', ...
    'MenuBar', 'none', ...
    'KeyPressFcn', 'autorefresh(handles)', ...
    'Tag', 'sigvar', ...
    'HandleVisibility','on', ...
    'Visible', 'of');
% Recover Position of previous SIGVAR Window.
if exist('pos_fig') == 1
    set(num_fig,'Position',pos_fig)
end
% Move following Round Screen to the upper left corner.
axes( ...
    'Units', 'normalized', ...
    'Position', [0.003, 0.27, 0.65, 0.65], ...
    'Visible', 'of', ...
    'NextPlot', 'replacechildren', ...
    'HandleVisibility', 'on')
% ============================================================
% Draw Round Screen.
num_rads = 04;                          % Number of radial circles.
num_sect = 12;                          % Number of sectors.
% - plot background (f.ck off POLAR command).
handles.pat = rectangle('Position',[-1, -1, +2, +2], 'Curvature',[1, 1]);
axis square
% - draw radial circles.
lin_styls = ['-'; repmat(':', num_rads-2, 1); '-'];
lin_colrs = [ 0 ; repmat(0.7, num_rads-2, 1);  0 ] * ones(1, 3);
hold on
for n = 1:num_rads-1
    rectangle(...
        'Position',[-n/num_rads -n/num_rads 2*n/num_rads 2*n/num_rads], ...
        'LineStyle',lin_styls(n), 'EdgeColor',lin_colrs(n,:), 'Curvature',[1, 1])
end
% - draw sectors.
theta = 2*pi*[1:num_sect]/num_sect;                 % Define a marks.
[x_part, y_part] = deal(cos(theta), sin(theta));    % Define marks X-Y position.
x_sect = [0.9*x_part/num_rads; x_part];             % Define Lines.
y_sect = [0.9*y_part/num_rads; y_part];
plot(x_sect,y_sect,':', 0,0,'o', 'Color',[0.7, 0.7, 0.7], ...
    'HandleVisibility','of')
% - draw area mark (i.e. dartboard).
combs_mat = repmat([1,  0,  0,  1, -1,  0,  0, -1], 2, 1);
coefs_mat = repmat([2/3; 3/2], 1, 8)/num_rads;
mark_mat = coefs_mat.*combs_mat;
plot(mark_mat(:,1:2:end),mark_mat(:,2:2:end),'k', 'HandleVisibility','of')
% - set Texts & Colours.
texts  = {'int8','int16','single','double'};  % Available Data Types (Precision).
colors = { 'b.' ,  'r.' ,   'g.' ,  'c.'  };  % Data Types Colors, respectively.
for n = 1:4                                   % Plot Points on top of grid.
    handles.pts(n) = plot(0, 0, colors{n});                 % Screen Points.
    handles.mks(n) = plot(-0.87, 1-(n-1)/8, colors{n});     % Legend Points.
end
hold off
% - set Fonts always on on top of Points.
for n = 1:4
    handles.txs(n) = text(0,0,texts{n}, 'FontSize',8);      % Screen Texts.
end
% ============================================================
% Configure UICONTROLS...   job of several weeks  :)
% - configure  VARIATOR_MODE section:
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'String', 'Variator Mode:', ...
    'FontSize', 10, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.82 0.923 0.18 0.032]);
handles.mode(1) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', ['set(gcbo,''Value'',1), ', ...
        'set(handles.mode(2),''Value'',0), ', ...
        'set(handles.mode(3),''Value'',0), ', ...
        'autorefresh(handles,''mode'')'], ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'radiobutton', ...
    'Value', 1, ...
    'String', 'manual', ...
    'FontSize', 10, ...
    'TooltipString', 'manual tuning the Properties', ...
    'Units', 'normalized', ...
    'Position', [0.84 0.865 0.14 0.04]);
handles.mode(2) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', ['set(handles.mode(1),''Value'',0), ', ...
        'set(gcbo,''Value'',1), ', ...
        'set(handles.mode(3),''Value'',0), ', ...
        'autorefresh(handles,''mode'')'], ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'radiobutton', ...
    'Value', 0, ...
    'String', 'magnetic', ...
    'FontSize', 10, ...
    'TooltipString', 'magnetic tuning the Properties', ...
    'Units', 'normalized', ...
    'Position', [0.84 0.815 0.14 0.04]);
handles.mode(3) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', ['set(handles.mode(1),''Value'',0), ', ...
        'set(handles.mode(2),''Value'',0), ', ...
        'set(gcbo,''Value'',1), ', ...
        'autorefresh(handles,''mode'')'], ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'radiobutton', ...
    'Value', 0, ...
    'String', 'optimal', ...
    'FontSize', 10, ...
    'TooltipString', 'optimal tuning the Properties', ...
    'Units','normalized', ...
    'Position', [0.84 0.765 0.14 0.04]);
% - Info of Signal Properties section creation:
handles.frame = uicontrol( ...
    'Parent', num_fig, ...
    'Style', 'frame', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.325 0.26 0.387]);
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', sprintf('Info of Signal Properties\nto be saved'), ...
    'String', 'Info String:', ...
    'FontSize', 8, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.86 0.7 0.095 0.04]);
handles.info_string = uicontrol( ...
    'Parent', num_fig, ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', sprintf('Info of Signal Properties\nto be saved'), ...
    'FontSize', 10, ...
    'Units', 'normalized', ...
    'Position', [0.7075 0.327 0.255 0.3825]);
% - configure  GET_STRING  PushButton:
handles.getstring = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''getstring'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', sprintf(['generate Info String\nof Signal Properties']), ...
    'String', 'Get String', ...
    'FontSize', 10, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.825 0.252 0.14 0.06]);
% - configure  DEL  PushButton:
handles.delstring = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''delstring'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', 'delete Info String', ...
    'String', 'Del String', ...
    'FontSize', 10, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.825 0.183 0.14 0.06]);
% - configure  LISTEN  PushButton:
handles.listen = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''listen'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', 'listen the Signal', ...
    'String', '>>', ...
    'FontSize', 12, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.915 0.325 0.05 0.045]);
% - configure  SAVE  PushButton:
handles.save = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''save'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', 'create  .INF  file', ...
    'String', 'Save', ...
    'FontSize', 12, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.825 0.114 0.14 0.06]);
% - configure  CLOSE  PushButton:
uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'closereq', ...
    'Style', 'pushbutton', ...
    'TooltipString', 'Exit SIGVAR', ...
    'String', 'Close', ...
    'FontSize', 12, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'HandleVisibility', 'of', ...
    'Position', [0.825 0.045 0.14 0.06]);
% - configure Signal Properties text/frame blocks:
uicontrol( ...
    'Parent', num_fig, ...
    'Style', 'frame', ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'HandleVisibility', 'of', ...
    'Position', [0.65 0.08 0.16 0.12]);
handles.text_sig_len_cur = uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'TooltipString', 'current Length of Signal', ...
    'HorizontalAlignment', 'left', ...
    'FontSize', 9, ...
    'Units', 'normalized', ...
    'Position', [0.655 0.14 0.15 0.04]);
handles.text_f_cur = uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'TooltipString', 'current Sampling Frequency', ...
    'HorizontalAlignment', 'left', ...
    'FontSize', 9, ...
    'Units', 'normalized', ...
    'Position', [0.655 0.09 0.15 0.04]);
% - configure  RESET  PushButton:
handles.reset = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', ['set(handles.exact,''Value'',0), ', ...
        'autorefresh(handles,''reset'')'], ...
    'Style', 'pushbutton', ...
    'TooltipString', sprintf('Scales:\nreset values'), ...
    'String', 'Reset', ...
    'FontSize', 10, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.512 0.252 0.09 0.06]);
% - configure  EXACT  ToggleButton:
handles.exact = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''exact'')', ...
    'Style', 'togglebutton', ...
    'TooltipString', sprintf('Scales:\nchange step'), ...
    'String', 'Exact', ...
    'FontSize', 9, ...
    'Value', 0, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.512 0.318 0.09 0.045]);
% - configure  AIM  PushButton:
handles.aim = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''aim'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', sprintf('Points:\ntake aim'), ...
    'String', 'Aim', ...
    'FontSize', 10, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.512 0.88 0.09 0.06]);
% - Increment  INDICATOR  section creation:
handles.indicator(1) = uicontrol( ...
    'Parent', num_fig, ...
    'Style', 'frame', ...
    'BackgroundColor', [0.9 0.9 0.9], ...
    'TooltipString', sprintf('better Point:\nstep to outside'), ...
    'Units', 'normalized', ...
    'Position', [0.512 0.83 0.028 0.045]);
handles.indicator(2) = uicontrol( ...
    'Parent', num_fig, ...
    'Style', 'frame', ...
    'Units', 'normalized', ...
    'TooltipString', sprintf('active Points:\nbetter color'), ...
    'Position', [0.541 0.83 0.031 0.045]);
handles.indicator(3) = uicontrol( ...
    'Parent', num_fig, ...
    'Style', 'frame', ...
    'BackgroundColor', [0.9 0.9 0.9], ...
    'TooltipString', sprintf('better Point:\nstep to center'), ...
    'Units', 'normalized', ...
    'Position', [0.573 0.83 0.028 0.045]);
% - configure  ROTATE  PushButtons:
handles.rotate_left = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''rotate_left'')', ...
    'ButtonDownFcn','autorefresh(handles,''rotate_left'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', sprintf('Points:\nrotate left'), ...
    'String', '<', ...
    'FontSize', 12, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.053 0.252 0.041 0.06]);
handles.rotate_right = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''rotate_right'')', ...
    'ButtonDownFcn','autorefresh(handles,''rotate_right'')', ...
    'Style', 'pushbutton', ...
    'TooltipString', sprintf('Points:\nrotate right'), ...
    'String', '>', ...
    'FontSize', 12, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.101 0.252 0.041 0.06]);
% - configure  LIFE  ToggleButton:
handles.life = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''life'')', ...
    'Style', 'togglebutton', ...
    'TooltipString', sprintf('Points:\nsimulate activity'), ...
    'String', 'L i f e', ...
    'FontSize', 9, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Value', 0, ...
    'Units', 'normalized', ...
    'Position', [0.053 0.318 0.09 0.045]);
% - configure  POINTS  Checkboxes:
handles.num_pts(1) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''checkbox01'')', ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'checkbox', ...
    'Value', 1, ...
    'TooltipString', sprintf(['Blue mark: integers\nint8 [-128...+127]\n', ...
        'or  uint8 [0...255]']), ...
    'Units', 'normalized', ...
    'Position', [0.053 0.903 0.032 0.032]);
handles.num_pts(2) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''checkbox02'')', ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'checkbox', ...
    'Value', 1, ...
    'TooltipString', sprintf('Red mark: integers\nint16 [-65536...+65535]'), ...
    'Units', 'normalized', ...
    'Position', [0.053 0.863 0.032 0.032]);
handles.num_pts(3) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''checkbox03'')', ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'checkbox', ...
    'Value', 1, ...
    'TooltipString', sprintf('Green mark:\nsingle precision'), ...
    'Units','normalized', ...
    'Position', [0.053 0.823 0.032 0.032]);
handles.num_pts(4) = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''checkbox04'')', ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'checkbox', ...
    'Value', 1, ...
    'TooltipString', sprintf('Cyan mark:\ndouble precision'), ...
    'Units', 'normalized', ...
    'Position', [0.053 0.783 0.032 0.032]);
% - configure  SLIDERS :
handles.slider_f = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''slider_f'')', ...
    'Style', 'slider', ...
    'BackgroundColor', 'white', ...
    'Value', 1/7, ...
    'Units', 'normalized', ...
    'Position', [0.052 0.15 0.55 0.05]);
handles.slider_sig_len = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''slider_sig_len'')', ...
    'Style', 'slider', ...
    'BackgroundColor', 'white', ...
    'Value', 2/9, ...
    'Units', 'normalized', ...
    'Position', [0.65 0.252 0.038 0.69]);
% - configure  FIX  ToggleButton:
handles.fix = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', ['str = ''<><''; indx = get(gcbo,''Value'')+1; ', ...
        'set(gcbo,''String'',str(indx:indx+1)); ', ...
        'autorefresh(handles,''stuck'')'], ...
    'Style', 'togglebutton', ...
    'String', '<>', ...
    'FontSize', 8, ...
    'TooltipString', 'fix scales proportionally', ...
    'Value', 0, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Units', 'normalized', ...
    'Position', [0.615 0.213 0.024 0.03]);
% - configure  F_S  Edit Windows:
handles.f_min = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles)', ...
    'Style', 'edit', ...
    'TooltipString', 'lower Sampling Frequency value, Hz', ...
    'FontSize', 12, ...
    'BackgroundColor', 'white', ...
    'Units', 'normalized', ...
    'Position', [0.052 0.08 0.1 0.05]);
handles.f_max = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles)', ...
    'Style', 'edit', ...
    'TooltipString', 'higher Sampling Frequency value, Hz', ...
    'FontSize', 12, ...
    'BackgroundColor', 'white', ...
    'Units', 'normalized', ...
    'Position', [0.502 0.08 0.1 0.05]);
handles.f_cur = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''new_f'')', ...
    'Style', 'edit', ...
    'TooltipString', 'current Sampling Frequency value, Hz', ...
    'FontSize', 12, ...
    'BackgroundColor', 'white', ...
    'Units', 'normalized', ...
    'Position', [0.277 0.08 0.1 0.05]);
% - configure  SIG_LEN  Edit Windows:
handles.sig_len_min = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles)', ...
    'Style', 'edit', ...
    'TooltipString', 'lower Length of Signal value, s', ...
    'FontSize', 12, ...
    'BackgroundColor', 'white', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.252 0.085 0.05]);
handles.sig_len_max = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles)', ...
    'Style', 'edit', ...
    'TooltipString', 'higher Length of Signal value, s', ...
    'FontSize', 12, ...
    'BackgroundColor', 'white', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.892 0.085 0.05]);
handles.sig_len_cur = uicontrol( ...
    'Parent', num_fig, ...
    'Callback', 'autorefresh(handles,''new_sig_len'')', ...
    'Style', 'edit', ...
    'TooltipString', 'current Length of Signal value, s', ...
    'FontSize', 12, ...
    'BackgroundColor', 'white', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.772 0.085 0.05]);
% - configure unuicontrolled Text Blocks:
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', 'lower possible Sampling Frequency, Hz', ...
    'String', 'Fs min, Hz', ...
    'FontSize', 9, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.052 0.03 0.1 0.04]);
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', 'higher possible Sampling Frequency, Hz', ...
    'String', 'Fs max, Hz', ...
    'FontSize', 9, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.502 0.03 0.1 0.04]);
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', 'current Sampling Frequency, Hz', ...
    'String', 'Fs, Hz', ...
    'FontSize', 9, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.277 0.03 0.1 0.04]);
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', 'higher possible Length of Signal, s', ...
    'String', 'Tsig max, s', ...
    'FontSize', 9, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.842 0.1 0.04]);
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', 'lower possible Length of Signal, s', ...
    'String', 'Tsig min, s', ...
    'FontSize', 9, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.202 0.1 0.04]);
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'Style', 'text', ...
    'HorizontalAlignment', 'left', ...
    'TooltipString', 'current Length of Signal, s', ...
    'String', 'Tsig, s', ...
    'FontSize', 9, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.705 0.72 0.1 0.04]);
% - configure Title:
uicontrol( ...
    'Parent', num_fig, ...
    'BackgroundColor', [0.8 0.8 0.8], ...
    'CreateFcn', 'autorefresh(handles,''reset'')', ...
    'Style', 'text', ...
    'String', 'Signal Properties Variator', ...
    'FontWeight', 'bold', ...
    'FontSize', 12, ...
    'HandleVisibility', 'of', ...
    'Units', 'normalized', ...
    'Position', [0.3 0.95 0.4 0.05]);
% ============================================================
% Uncover the Variator (set it VISIBLE).
set(num_fig,'Visible','on')